// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

/// {@template theme_extension_provider}
/// A provider of [ThemeExtension]s to automatically apply to a [Theme].
/// {@endtemplate}
abstract class ThemeExtensionProvider {
  /// {@macro theme_extension_provider}
  const ThemeExtensionProvider();

  /// Returns the [ThemeExtension]s to apply to the current [ThemeData].
  List<ThemeExtension<dynamic>> getThemeExtensionsFor(ThemeData theme);

  /// Returns a [ThemeData] with the [ThemeExtension]s applied.
  ThemeData applyExtensionsTo(
    ThemeData theme,
  ) =>
      theme.copyWith(
        extensions: [
          ...theme.extensions.values,
          ...getThemeExtensionsFor(theme),
        ],
      );
}

/// {@template default_theme_extension_provider}
/// A [ThemeExtensionProvider] that provides the default [ThemeExtension]s
/// for the current [ThemeData.brightness].
/// {@endtemplate}
class DefaultThemeExtensionProvider extends ThemeExtensionProvider {
  /// {@macro default_theme_extension_provider}
  const DefaultThemeExtensionProvider();

  /// Default light theme extensions form Wyatt UI Components
  ///
  /// This needs to be added to the [ThemeData] in order to use the
  /// Wyatt UI Components widgets.
  ///
  /// Those extensions are the default ones used in fallback mode
  /// when no custom theme is provided.
  ///
  /// See `wyatt_ui_kit` for a custom theme example.
  static List<ThemeExtension<dynamic>> get defaultLight => [
        // Cards
        CardThemeExtensionDefault.light(),
        // Loader
        LoaderThemeExtensionDefault.light(),
        // RichText
        RichTextBuilderThemeExtensionDefault.light(),
        // Buttons
        FileSelectionButtonThemeExtensionDefault.light(),
        FlatButtonThemeExtensionDefault.light(),
        SimpleIconButtonThemeExtensionDefault.light(),
        SymbolButtonThemeExtensionDefault.light(),
        // TextInput
        TextInputThemeExtensionDefault.light(),
        // TopBar
        TopBarThemeExtensionDefault.light(),
      ];

  /// Default dark theme extensions form Wyatt UI Components
  ///
  /// This needs to be added to the [ThemeData] in order to use the
  /// Wyatt UI Components widgets.
  ///
  /// Those extensions are the default ones used in fallback mode
  /// when no custom theme is provided.
  ///
  /// See `wyatt_ui_kit` for a custom theme example.
  static List<ThemeExtension<dynamic>> get defaultDark => [
        // Cards
        CardThemeExtensionDefault.dark(),
        // Loader
        LoaderThemeExtensionDefault.dark(),
        // RichText
        RichTextBuilderThemeExtensionDefault.dark(),
        // Buttons
        FileSelectionButtonThemeExtensionDefault.dark(),
        FlatButtonThemeExtensionDefault.dark(),
        SimpleIconButtonThemeExtensionDefault.dark(),
        SymbolButtonThemeExtensionDefault.dark(),
        // TextInput
        TextInputThemeExtensionDefault.dark(),
        // TopBar
        TopBarThemeExtensionDefault.dark(),
      ];

  @override
  List<ThemeExtension<dynamic>> getThemeExtensionsFor(ThemeData theme) =>
      theme.brightness == Brightness.light ? defaultLight : defaultDark;
}
